/* Pika - Photo and Image Kooker Application
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "libpikabase/pikabase.h"

#include "core-types.h"

#include "pikaimage.h"
#include "pikalayer.h"
#include "pikalayerpropundo.h"


static void   pika_layer_prop_undo_constructed (GObject             *object);

static void   pika_layer_prop_undo_pop         (PikaUndo            *undo,
                                                PikaUndoMode         undo_mode,
                                                PikaUndoAccumulator *accum);


G_DEFINE_TYPE (PikaLayerPropUndo, pika_layer_prop_undo, PIKA_TYPE_ITEM_UNDO)

#define parent_class pika_layer_prop_undo_parent_class


static void
pika_layer_prop_undo_class_init (PikaLayerPropUndoClass *klass)
{
  GObjectClass  *object_class = G_OBJECT_CLASS (klass);
  PikaUndoClass *undo_class   = PIKA_UNDO_CLASS (klass);

  object_class->constructed = pika_layer_prop_undo_constructed;

  undo_class->pop           = pika_layer_prop_undo_pop;
}

static void
pika_layer_prop_undo_init (PikaLayerPropUndo *undo)
{
}

static void
pika_layer_prop_undo_constructed (GObject *object)
{
  PikaLayerPropUndo *layer_prop_undo = PIKA_LAYER_PROP_UNDO (object);
  PikaLayer         *layer;

  G_OBJECT_CLASS (parent_class)->constructed (object);

  pika_assert (PIKA_IS_LAYER (PIKA_ITEM_UNDO (object)->item));

  layer = PIKA_LAYER (PIKA_ITEM_UNDO (object)->item);

  switch (PIKA_UNDO (object)->undo_type)
    {
    case PIKA_UNDO_LAYER_MODE:
      layer_prop_undo->mode            = pika_layer_get_mode (layer);
      layer_prop_undo->blend_space     = pika_layer_get_blend_space (layer);
      layer_prop_undo->composite_space = pika_layer_get_composite_space (layer);
      layer_prop_undo->composite_mode  = pika_layer_get_composite_mode (layer);
      break;

    case PIKA_UNDO_LAYER_OPACITY:
      layer_prop_undo->opacity = pika_layer_get_opacity (layer);
      break;

    case PIKA_UNDO_LAYER_LOCK_ALPHA:
      layer_prop_undo->lock_alpha = pika_layer_get_lock_alpha (layer);
      break;

    default:
      g_return_if_reached ();
    }
}

static void
pika_layer_prop_undo_pop (PikaUndo            *undo,
                          PikaUndoMode         undo_mode,
                          PikaUndoAccumulator *accum)
{
  PikaLayerPropUndo *layer_prop_undo = PIKA_LAYER_PROP_UNDO (undo);
  PikaLayer         *layer           = PIKA_LAYER (PIKA_ITEM_UNDO (undo)->item);

  PIKA_UNDO_CLASS (parent_class)->pop (undo, undo_mode, accum);

  switch (undo->undo_type)
    {
    case PIKA_UNDO_LAYER_MODE:
      {
        PikaLayerMode          mode;
        PikaLayerColorSpace    blend_space;
        PikaLayerColorSpace    composite_space;
        PikaLayerCompositeMode composite_mode;

        mode            = pika_layer_get_mode (layer);
        blend_space     = pika_layer_get_blend_space (layer);
        composite_space = pika_layer_get_composite_space (layer);
        composite_mode  = pika_layer_get_composite_mode (layer);

        pika_layer_set_mode            (layer, layer_prop_undo->mode,
                                        FALSE);
        pika_layer_set_blend_space     (layer, layer_prop_undo->blend_space,
                                        FALSE);
        pika_layer_set_composite_space (layer, layer_prop_undo->composite_space,
                                        FALSE);
        pika_layer_set_composite_mode  (layer, layer_prop_undo->composite_mode,
                                        FALSE);

        layer_prop_undo->mode            = mode;
        layer_prop_undo->blend_space     = blend_space;
        layer_prop_undo->composite_space = composite_space;
        layer_prop_undo->composite_mode  = composite_mode;
      }
      break;

    case PIKA_UNDO_LAYER_OPACITY:
      {
        gdouble opacity;

        opacity = pika_layer_get_opacity (layer);
        pika_layer_set_opacity (layer, layer_prop_undo->opacity, FALSE);
        layer_prop_undo->opacity = opacity;
      }
      break;

    case PIKA_UNDO_LAYER_LOCK_ALPHA:
      {
        gboolean lock_alpha;

        lock_alpha = pika_layer_get_lock_alpha (layer);
        pika_layer_set_lock_alpha (layer, layer_prop_undo->lock_alpha, FALSE);
        layer_prop_undo->lock_alpha = lock_alpha;
      }
      break;

    default:
      g_return_if_reached ();
    }
}
